unit ContRingTestUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, ExtCtrls, ImgList;

type
  TForm1 = class(TForm)
    butStart: TButton;
    ADCTime: TTimer;
    butEnd: TButton;
    butLog: TButton;
    lbStatus: TListBox;
    tbRate: TTrackBar;
    labRate: TLabel;
    Label1: TLabel;
    GroupBox1: TGroupBox;
    LevelImage0: TImage;
    LevelImage1: TImage;
    LevelImage2: TImage;
    LevelImage3: TImage;
    LevelImage4: TImage;
    LevelImage5: TImage;
    LevelImage6: TImage;
    LevelImage7: TImage;
    DIORadioIgnore: TRadioButton;
    DIORadioInput: TRadioButton;
    DIORadioOutput: TRadioButton;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    Label9: TLabel;
    Label10: TLabel;
    LevelImages: TImageList;
    DIOReadTime: TTimer;
    GroupBox2: TGroupBox;
    DACLabel0: TLabel;
    DACLabel1: TLabel;
    DACTrack0: TTrackBar;
    DACTrack1: TTrackBar;
    Label11: TLabel;
    Label12: TLabel;
    Setup31kButton: TButton;
    Setup25kButton: TButton;
    SetupTestButton: TButton;
    ResultPaint: TPaintBox;
    ScanLabel: TLabel;
    procedure butStartClick(Sender: TObject);
    procedure ADCTimeTimer(Sender: TObject);
    procedure butEndClick(Sender: TObject);
    procedure tbRateChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure DIORadioClick(Sender: TObject);
    procedure LevelImageMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure DIOReadTimeTimer(Sender: TObject);
    procedure DACTrack0Change(Sender: TObject);
    procedure DACTrack1Change(Sender: TObject);
    procedure tbPollRateChange(Sender: TObject);
    procedure Setup31kButtonClick(Sender: TObject);
    procedure Setup25kButtonClick(Sender: TObject);
    procedure SetupTestButtonClick(Sender: TObject);
    procedure ResultPaintPaint(Sender: TObject);
  protected
    function HandleStatus(Status: Longword; msg: String = 'Error: %u'): Boolean;
    procedure DrawDIOData(Changes: LongWord);
    { Private declarations }
  public
    { Public declarations }
  end;

{
object Label2: TLabel
  Left = 240
  Top = 128
  Width = 447
  Height = 33
  Anchors = [akLeft, akTop, akRight]
  AutoSize = False
  Caption = 'Use this slider to set rate of polling timer.'
  WordWrap = True
end
object labPollRate: TLabel
  Left = 440
  Top = 128
  Width = 239
  Height = 16
  Anchors = [akLeft, akTop, akRight]
  AutoSize = False
  Caption = 'labRate'
end
object tbPollRate: TTrackBar
  Left = 240
  Top = 144
  Width = 447
  Height = 41
  Anchors = [akLeft, akTop, akRight]
  Max = 66
  Min = 1
  Orientation = trHorizontal
  PageSize = 100
  Frequency = 100
  Position = 66
  SelEnd = 0
  SelStart = 0
  TabOrder = 6
  ThumbLength = 25
  TickMarks = tmBottomRight
  TickStyle = tsAuto
  OnChange = tbPollRateChange
end
}

var
  Form1: TForm1;

implementation

{$R *.DFM}

uses
  Math, AIOUSB;

const
  rdUSE10 = 0;  rdBSE10 = 1;// 0-10V, 10V
  rdUSE5 = 2;   rdBSE5 = 3; // 0-5V, 5V
  rdUSE2 = 4;   rdBSE2 = 5; // 0-2V, 2V
  rdUSE1 = 6;   rdBSE1 = 7; // 0-1V, 1V
  rdUDif10 = rdUSE10 or $80; rdBDif10 = rdBSE10 or $80; // differential is configured by setting the MSB (bit 7)
  rdUDif5  = rdUSE5  or $80; rdBDif5  = rdBSE5  or $80;
  rdUDif2  = rdUSE2  or $80; rdBDif2  = rdBSE2  or $80;
  rdUDif1  = rdUSE1  or $80; rdBDif1  = rdBSE1  or $80;

type
  SetConfigStruct =
    packed record
      Gains: array[0..15] of Byte;
      CalCode: Byte;
      Trigger: Byte;
      StartEnd: Byte;
      Oversample: Byte;
      ExtStartEnd: Byte;
    end;
var
  ADCConfiguration: SetConfigStruct;
  ADCPassCount: Integer;
  LevelImage: array[0..7] of TImage;
  LowLevel, HighLevel: TBitmap;
  DIOData: LongWord;
  bForceDIOImageUpdate: Boolean;
  pfHz : Int64;
  pfNow, pfLast : Int64;
  pf2ms: double;
  timeoutCounter : Int64;
  TimerScansToRead: LongWord = 2;
  TimerOversample: LongWord = 0;
  NumChannels: LongWord = 16;
  PacketsPerBlock: LongWord = 0;
  ResultLine: TStringList;

type
  TAIOErrorStruct = record
    Success: Boolean;
    Warning: Boolean;
    Error: Boolean;
    Code: Int64;
    Msg: ShortString;
  end;

{
function ADC_CalcBytePerSecond( DeviceID: Word;
                                Chans:Byte; Oversamples: Byte;
                                ScanStart: Boolean;
                                TargetHz:Double; var ActualHz: Double;
                                var BytesPerSecond: Longword
                              ):TAIOErrorStruct;
  var
    Speed, bps: double;
begin
  case ((DeviceID and $1F) mod 5) of
    0: Speed := 500000.0; // 16-bit "A" models
    1: Speed := 250000.0; // 16-bit "E" models
    2: Speed := 500000.0; // 12-bit "A" models
    3: Speed := 250000.0; // 12-bit " " models
    4: Speed := 100000.0; // 12-bit "E" models
  end;

  bps := TargetHz;
  if ScanStart then bps := bps * Chans * (Oversamples+1);
  if bps > Speed then
  begin
    bps := Speed;
    ActualHz := Speed / ( TargetHz ) * (ord(ScanStart) * Chans * (Oversamples+1));
  end;
  bps := bps * 2; // convert to bytes
end;
}

// max USB packet call rate is FAST
// ADC_ReadData is linear: 256 readings consume about 25usec
// MAX USB bandwidth is ~20MHz (at least the part we might care about at the moment)
// desired data latency is in three domains: 1: "all about the latency", 2: "human vision convenient", and 3: "all about the bandwidth"
// where 1: is "as fast as the bus can go" or about 1kHz in USB, 2: is 3-60Hz, and 3: is very slow, maybe 1second or 0.1/minute even.

//



{
function CalcTimings(
                      var StreamingBlockSize: Integer;       // causes latency
                      var TotalBytesPerSecond: Integer;      // bandwidth consumed
                      var ResponsivenessTargetHz: double;  // target slowest rate for data to be received by application
                      var TimeoutSeconds: double;          // preferred/recommended Timeout in Seconds
                      WhatToOptimizeFor: byte
                    ) : TAIOErrorStruct;
  const
    SecondsPer512 = 25E-6; // 25 microseconds

   function MinStreamingBlockSize( TBpS: Integer; Hz: double ): Integer;
   begin
     Result := max(512, Floor((TBpS / Hz) / 512) * 512);
   end;

   function MaxResponsivenessTargetHz( SBS, TotalBytesPerSecond: Integer ): double;
   begin
     Result := TotalBytesPerSecond / SBS;
   end;

   Function MaxBytesPerSecond(  ): Integer;
   begin
   end;

begin
  result.Success := False;
  result.Warning:= False;
  result.Error := False;
  result.Msg := 'Successful';

  if WhatToOptimizeFor = 1 then StreamingBlockSize := 512;
  if WhatToOptimizeFor = 2 then
  begin
    if (ResponsivenessTargetHz >= 3.0) and (ResponsivenessTargetHz <= 60.0) then
      StreamingBlockSize := Floor((TotalBytesPerSecond / ResponsivenessTargetHz) / 512) * 512;

    if ResponsivenessTargetHz < 3.0 then
    begin
      //if
    end;

    if ResponsivenessTargetHz < 60.0 then
    begin
    end;

  end;
  if WhatToOptimizeFor = 3 then


end;
}

procedure TForm1.FormCreate(Sender: TObject);
begin
  LowLevel := TBitmap.Create;
  HighLevel := TBitmap.Create;
  ResultLine := TStringList.Create;

  LevelImage[ 0] := LevelImage0;
  LevelImage[ 1] := LevelImage1;
  LevelImage[ 2] := LevelImage2;
  LevelImage[ 3] := LevelImage3;
  LevelImage[ 4] := LevelImage4;
  LevelImage[ 5] := LevelImage5;
  LevelImage[ 6] := LevelImage6;
  LevelImage[ 7] := LevelImage7;

  LevelImages.GetBitmap(0, LowLevel);
  LevelImages.GetBitmap(1, HighLevel);

  tbRateChange(tbRate);
  //tbPollRateChange(tbPollRate);
  QueryPerformanceFrequency(pfHz);
  pf2ms := 1000/pfHz;
  QueryPerformanceCounter(pfNow);
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  FreeAndNil(LowLevel);
  FreeAndNil(HighLevel);
  FreeAndNil(ResultLine);
end;

function TForm1.HandleStatus(Status: Longword; msg:String='Error: %u'): Boolean;
begin
  Result := True;
  if Status = ERROR_TIMEOUT then exit;
  if Status <> ERROR_SUCCESS then
  begin
    Result := False;
    lbStatus.Items.Add('Operation ended due to error '+IntToStr(Status) + ' @ ' + DateTimeToStr(Now));
    Exit;
  end;
end;

procedure TForm1.butStartClick(Sender: TObject);
  var
    ADCHz: Double;
    L: Longword;
    i: integer;
    deltaMillsec : double;
begin
  for i:=0 to 31 do AIOUSB_CloseDevice(i);
  GetDevices;

  pfLast := pfNow;
  QueryPerformanceCounter(pfNow);
  deltaMillsec := (pfNow-pfLast) * pf2ms;

  with ADCConfiguration do begin
    for i:= 0 to 15 do Gains[i] := rdBSE10;
    CalCode := 0;
    Trigger := 5;
    StartEnd := (NumChannels - 1) * $10; //Assumes max NumChannels = 16.
    Oversample := TimerOversample;
    ExtStartEnd := $00;
  end;

  L  :=  sizeof(ADCConfiguration);
  ADCHz := tbRate.Position;
  if not HandleStatus( ADC_FullStartRing(diOnly, @ADCConfiguration, L, ':NONE:', @ADCHz, 64*1024*1024, PacketsPerBlock), 'ADC_FullStartRing: %u' ) then exit;
  lbStatus.Items.Add('Operation Started @ ' + DateTimeToStr(Now) + ' after ' + FloatToStrF(deltaMillsec, ffFixed, 5, 2) + ' ms');

  ScanLabel.Caption := '';
  ADCPassCount := 0;
  butEnd.Enabled := True;
  butStart.Enabled := False;
  tbRate.Enabled := False;
  ADCTime.Enabled := True;
  Setup31kButton.Enabled := False;
  Setup25kButton.Enabled := False;
  SetupTestButton.Enabled := False;
end;

procedure TForm1.ADCTimeTimer(Sender: TObject);
const
  TimeoutSeconds = 2/5;

var
  ADCData: array of Double;
  ScansToRead: Longword;
  Channel, Scan: Integer;
  deltaMillsec: Double;
  Status: Longword;

begin
  if not ADCTime.Enabled then Exit;

  pfLast := pfNow;
  QueryPerformanceCounter(pfNow);
  deltaMillsec := (pfNow-pfLast) * pf2ms;

  ResultLine.Clear;
  ScansToRead := TimerScansToRead;
  SetLength(ADCData, ScansToRead * NumChannels);

  Status := ADC_ReadData(diOnly, @ADCConfiguration, ScansToRead, @ADCData[0], TimeoutSeconds);
  if Status = ERROR_TIMEOUT then begin
    inc(timeoutCounter);
    ResultLine.Add('Timeout: ' + IntToStr(timeoutCounter));
  end
  else if Status = ERROR_SUCCESS then begin
    timeoutCounter := 0;
    {
    if ScansToRead = TimerScansToRead then
      ScanLabel.Caption := 'MAX scans'
    else
    }
      ScanLabel.Caption := IntToStr(ScansToRead) + '/' + IntToStr(TimerScansToRead) + ' scans'
    ;
    {
    if TimerScansToRead < 3 then begin
      for scan := 0 to ScansToRead-1 do begin
        for channel := 0 to NumChannels-1 do begin
          ResultLine.Add(Format('CH %-2d : %-8.4f', [ channel, ADCData[scan * Integer(NumChannels) + channel] ]) );
        end;
        ResultLine.Add('');
      end;
    end;
    }
    Scan := ScansToRead - 1;
    for channel := 0 to NumChannels-1 do begin
      ResultLine.Add('Ch ' + IntToStr(Channel) + ': ' + IntToStr(Round(1000 * ADCData[scan * Integer(NumChannels) + channel])) + ' mV');
    end;
    ResultLine.Add('');
  end;// end else-if

  inc(ADCPassCount);
  ResultLine.Add(IntToStr(ADCPassCount) + ' polls @ '+FloatToStrF(deltaMillsec, ffFixed, 5, 1)+' ms delay');
  if 50 * Round(1/50 * deltaMillsec) = 200 then ResultLine.Add('Caught Up');
  ResultPaintPaint(ResultPaint);
end;

procedure TForm1.butEndClick(Sender: TObject);
begin
  ADCTime.Enabled := False;
  lbStatus.Items.Add('Operation Ended by User @ ' +
                      DateTimeToStr(Now)
                    );
  lbStatus.Items.Add('  -- after ' +
                      IntToStr(ADCPassCount) +
                     ' calls to ADC_ReadData().'
                    );
  butEnd.Enabled := False;
  butStart.Enabled := True;
  tbRate.Enabled := True;
  Setup31kButton.Enabled := True;
  Setup25kButton.Enabled := True;
  SetupTestButton.Enabled := True;
  if not HandleStatus(ADC_BulkContinuousEnd(diOnly, nil), 'ADC_BulkContinuousEnd: %u') then exit;
end;

procedure TForm1.tbRateChange(Sender: TObject);
begin
  labRate.Caption := IntToStr(tbRate.Position) + '.0 Hz ADC Starts';
  // only takes effect the next time you issue a start
end;

procedure TForm1.tbPollRateChange(Sender: TObject);
begin
  //ADCTime.Interval := ceil (1/tbPollRate.Position * 1000); // rate in ms
  //labPollRate.Caption := IntToStr(tbPollRate.Position) + '.0 Hz ADC_ReadData';

end;

procedure TForm1.DIORadioClick(Sender: TObject);
var
  DIOOuts: Byte;
  I: Integer;

begin
  if DIORadioInput.Checked then begin
    DIOOuts := $0;
    DIO_ConfigureMasked(diOnly, @DIOOuts, nil, nil, nil, nil, nil);
    bForceDIOImageUpdate := True;

    DIOReadTime.Enabled := True;
    DIOReadTimeTimer(DIOReadTime);
  end
  else if DIORadioOutput.Checked then begin
    DIOReadTime.Enabled := False;

    DIOOuts := $F;
    DIOData := $FFFFFFFF; //This sample defaults to high, because a pullup is weaker than a ground.
    DIO_ConfigureMasked(diOnly, @DIOOuts, nil, @DIOData, nil, nil, nil);

    DrawDIOData($FFFFFFFF);
  end
  else begin
    DIOReadTime.Enabled := False;

    for I := 0 to 7 do
      LevelImage[I].Picture.Assign(nil)
    ;
  end;
end;

procedure TForm1.LevelImageMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  I: Integer;
  
begin
  if not DIORadioOutput.Checked then Exit;

  I := (Sender as TImage).Tag;
  //Toggle that bit.
  DIOData := DIOData xor (1 shl I);
  //Output the new state.
  DIO_WriteAll(diOnly, @DIOData);
  //Redraw that bit.
  DrawDIOData(1 shl I);
end;

procedure TForm1.DIOReadTimeTimer(Sender: TObject);
var
  oDIOData, Changes: LongWord;

begin
  oDIOData := DIOData;
  DIO_ReadAll(diOnly, @DIOData);
  Changes := DIOData xor oDIOData;

  if bForceDIOImageUpdate then begin
    Changes := $FFFFFFFF;
    bForceDIOImageUpdate := False;
  end;

  DrawDIOData(Changes);
end;

procedure TForm1.DrawDIOData(Changes: LongWord);
var
  I: Integer;

begin
  for I := 0 to 7 do
    if (Changes and (1 shl I)) <> 0 then begin
      if (DIOData and (1 shl I)) <> 0 then
        LevelImage[I].Picture.Assign(HighLevel)
      else
        LevelImage[I].Picture.Assign(LowLevel)
    end
  ;
end;

procedure TForm1.DACTrack0Change(Sender: TObject);
var
  Counts: Word;

begin
  Counts := DACTrack0.Position;
  DACDirect(diOnly, 0, Counts);
  DACLabel0.Caption := 'DAC 0: ' + IntToStr(Round(100/65536 * Counts)) + '% [' + IntToHex(Counts, 4) + ']';
end;

procedure TForm1.DACTrack1Change(Sender: TObject);
var
  Counts: Word;

begin
  Counts := DACTrack1.Position;
  DACDirect(diOnly, 1, Counts);
  DACLabel1.Caption := 'DAC 1: ' + IntToStr(Round(100/65536 * Counts)) + '% [' + IntToHex(Counts, 4) + ']';
end;


procedure TForm1.Setup31kButtonClick(Sender: TObject);
begin
  tbRate.Position := 30722;
  TimerScansToRead := 6144;
  NumChannels := 3;
  TimerOversample := 0;
  PacketsPerBlock := 6;
end;

procedure TForm1.Setup25kButtonClick(Sender: TObject);
begin
  tbRate.Position := 25000;
  TimerScansToRead := 5000;
  NumChannels := 9;
  TimerOversample := 0;
  PacketsPerBlock := 5;
end;

procedure TForm1.SetupTestButtonClick(Sender: TObject);
begin
  tbRate.Position := 360;
  TimerScansToRead := 5000;
  NumChannels := 9;
  TimerOversample := 0;
  PacketsPerBlock := 5;
end;

procedure TForm1.ResultPaintPaint(Sender: TObject);
var
  H, I: Integer;
  R: TRect;

begin
  R := ResultPaint.ClientRect;
  H := R.Bottom;
  ResultPaint.Canvas.Brush.Color := clWindow;
  ResultPaint.Canvas.Font.Color := clWindowText;
  for I := 0 to ResultLine.Count-1 do begin
    R.Bottom := 26 + 16*I;
    ResultPaint.Canvas.TextRect(R, 10, R.Bottom - 16, ResultLine[I]);
    R.Top := R.Bottom;
  end;
  R.Bottom := H;
  ResultPaint.Canvas.FillRect(R);
end;

end.
